import React, { useEffect, useState } from "react";
import { useRouter } from 'next/router'
import { fetchServices, ServiceType, Tag } from '@/lib/contentful'
import { removeFromObjRecursive, replaceEmailsRecursive } from '@/lib/utils'
import { documentToHtmlString } from '@contentful/rich-text-html-renderer'
import { IconArrowBigRight, IconChevronLeft, IconMail, IconBooks, IconBook2 } from '@tabler/icons'

import Tooltip from '@/components/Tooltip'
import ServiceTag from "@/components/ServiceTag";
import Chip from "@/components/Chip";
import Link from "next/link"
import { encodeEmail, decodeEmail, handleMailtoEvent } from '@/lib/utils';

import filterData from '@/lib/assets/data/currentFilter.json';

export default function ({ services }) {
  const router = useRouter()
  const filters = filterData.filter;
  const defaultFilters: any = {
    type: [],
    status: [],
  }
  const [activeFilters, setActiveFilters] = useState(defaultFilters)
  const [filteredServices, setFilteredServices] = useState(services)

  useEffect(() => {
    // filter param change, update filter and filter servicves
    if (router.query.filter) {
      setActiveFilters(paramsToFilter(router.query.filter))
    } else {
      setActiveFilters(defaultFilters)
    }
    setFilteredServices(filterServices())
  }, [router.query.filter])

  function filterToParams() {
    const res = []
    Object.keys(activeFilters).forEach(f => {
      if (activeFilters[f].length > 0) {
        activeFilters[f].forEach(i => res.push(i))
      }
    })
    return [...new Set(res)].join('&')
  }

  function paramsToFilter(str) {
    if (!str) return defaultFilters
    const filters = str.split('&')
    filters.forEach(f => {
      const filter = f.split(':')[0]
      if (activeFilters[filter].indexOf(f) < 0) {
        activeFilters[filter].push(f)
      }
    })
    return activeFilters
  }

  function filterServices() {
    return services.filter((s: ServiceType) => {
      const tags: string[] = s.tags ? s.tags.map(o => o.fields ? o.fields.name : '') : []
      const groups: any = {}
      let match = true

      tags.forEach(tag => {
        const t = tag.split(':')
        if (!groups[t[0]]) groups[t[0]] = []
        groups[t[0]].push(tag)
      })

      Object.keys(activeFilters).forEach(f => {
        if (activeFilters[f].length > 0) {
          if (activeFilters[f].length === 0) return
          if (!groups[f]) return match = false
          groups[f].forEach(item => {
            if (item.startsWith(f + ':') && !activeFilters[f].includes(item)) {
              match = false
            }
          })
        }
      })

      return match
    })
  }

  function handleFilters(event) {
    const filter = event.target.id.split('-')[0]
    if (event.target.checked) {
      activeFilters[filter].push(event.target.value)
    } else if (activeFilters[filter].indexOf(event.target.value) >= 0) {
      activeFilters[filter].splice(activeFilters[filter].indexOf(event.target.value), 1)
    }
    // we always track filters in query params, param change will trigger filtering
    router.push(`?filter=${encodeURIComponent(filterToParams())}`, undefined, { shallow: true })
  }

  const hasTag = (serviceType: ServiceType, tagName: string) => {
    const tags: string[] = serviceType.tags ? serviceType.tags.filter(tag => tag.fields && tag.fields.name === tagName) : []
    return tags.length > 0
  }

  return (
    <div className='container'>
      <nav
        className="relative max-w-7xl mx-auto flex items-center justify-between px-3 sm:px-6 "
      >
        <div className="flex items-center flex-1">
          <div className="flex items-center justify-between w-full md:w-auto">
            <div className="lg:block md:space-x-10 hidden">
              <Link href="/" passHref>
                <a className={'text-lg font-bold text-yellow-400 hover:text-gray-900'}>
                  Föderales Entwicklungsportal
                </a>
              </Link>
            </div>
          </div>
        </div>
      </nav>

      <div className="py-10">
        <div className="max-w-2xl mx-auto px-3 sm:px-6 lg:px-8 lg:max-w-7xl lg:grid lg:grid-cols-12 lg:gap-8">
          <header className="lg:col-start-3 xl:col-start-3 lg:col-span-9 xl:col-span-9 flex flex-row">
          <div className="flex lg:hidden">
          <Link href="/" passHref>
            <a>
              <IconChevronLeft
                className="mr-7 mt-6 text-yellow-400 hover:text-gray-900"
                aria-hidden="true"
              />
            </a>
          </Link>
          </div>
            <h1>
              <span className="overflow-hiddenmt-1 block text-4xl tracking-tight font-extrabold sm:text-5xl xl:text-7xl">
                <span className="block text-gray-900">Verfügbare Plattformen &amp; Basisdienste</span>
              </span>
            </h1>
          </header>
          <form name="filters" className="lg:block lg:col-span-2 xl:col-span-2">
            <nav
              aria-label="Sidebar"
              className="sticky top-4 divide-y divide-gray-300"
            >
              {filters.map((filter) => (
                <div className="pt-6 mt-4" key={filter.id}>
                  <p
                    className="text-xs font-semibold text-gray-500 uppercase tracking-wider"
                    id={`${filter.id}-headline`}
                  >
                    {filter.label}
                  </p>
                  <div
                    className="pl-3 mt-3 space-y-2"
                    aria-labelledby={`${filter.id}-headline`}
                  >
                    {filter.options.map((option) => (
                      <div key={option.value} className="flex items-center">
                        <input
                          onClick={handleFilters}
                          id={`${filter.id}-${option.id}`}
                          name={`${filter.id}[]`}
                          defaultValue={`${filter.id}:${option.id}`}
                          defaultChecked={router.query.filter ? router.query.filter.includes(`${filter.id}:${option.id}`) : false}
                          type="checkbox"
                          className="h-4 w-4 border-gray-300 rounded text-indigo-600 focus:ring-indigo-500"
                        />
                        <label
                          htmlFor={`${filter.id}-${option.id}`}
                          className="ml-3 text-sm text-gray-500"
                        >
                          {option.label}
                        </label>
                      </div>
                    ))}
                  </div>
                </div>
              ))}
            </nav>
          </form>
          <main className="lg:col-start-3 xl:col-start-3 lg:col-span-9 xl:col-span-9">
            <div className="mt-4">
              <ul role="list" className="space-y-4">
                {filteredServices.map((service : any, key) => (
                  <li
                    key={service.slug}
                    className="bg-white px-4 pt-3 pb-6 shadow sm:p-6 sm:rounded-lg"
                  >
                    <div className="text-right md:float-right">
                      {hasTag(service, 'label:external') ?
                        <Tooltip
                          delayIn={200}
                          delayOut={600}
                          content="Bei diesem Inhalt handelt es sich um einen externen Inhalt, der keiner Qualitätssicherung seitens der FITKO unterliegt."
                        >
                          <Chip text={'externer Inhalt'} style={'bg-indigo-500'} />
                        </Tooltip>
                      : ''}
                    </div>
                    <div aria-labelledby={'service-' + key}>
                      <div className="flex justify-between">
                        <div className="flex space-x-3">
                          <div className="flex-shrink-0">
                          {service?.logo ?(
                            <img
                              className="max-w-[100px] h-10"
                              src={service?.logo?.fields.file.url}
                              alt={service?.logo?.fields.name}
                            />
                            ) : (
                            <IconBooks
                                className={'h10'}
                              />
                          )}
                          </div>
                          <div className="min-w-0 flex-1">
                            <p className="text-lg font-medium text-gray-900">
                              <Link href={`/resources/${service.slug}`} passHref>
                                <a className="hover:underline">
                                  {service.name}
                                </a>
                              </Link>
                            </p>
                          </div>
                        </div>
                      </div>
                      <div className="container flex flex-col md:flex-row my-2">
                        {service?.tags?.map((tag : Tag, i: number) => {
                          if (tag.fields && tag.fields.name) {
                            return <ServiceTag key={i} text={tag.fields.name} style={'bg-yellow-400'} />
                          }
                        })}
                      </div>
                      <div
                        className="mt-2 text-sm text-gray-700 space-y-4"
                        dangerouslySetInnerHTML={{
                          __html: documentToHtmlString(service.shortDescription),
                        }}
                      />
                      <div className="mt-6 flex justify-end space-x-8">
                        <div className="flex text-sm">
                          <span className="inline-flex items-center text-sm mx-5 xl:mx-12">
                            <Link href={`${service.documentation}`} passHref>
                              <a className="inline-flex space-x-2 text-gray-400 hover:text-gray-800">
                                <span className="font-medium">Direkt zur Ressource</span>
                                <IconBook2
                                  className="h-5 w-5"
                                  aria-hidden="true"
                                />
                              </a>
                            </Link>
                          </span>
                          <span className="inline-flex items-center text-sm">
                            <Link href={`/resources/${service.slug}`} passHref>
                              <a className="inline-flex space-x-2 text-gray-400 hover:text-gray-800">
                                <span className="font-medium">Mehr</span>
                                <IconArrowBigRight
                                  className="h-5 w-5"
                                  aria-hidden="true"
                                />
                              </a>
                            </Link>
                          </span>
                        </div>
                      </div>
                    </div>
                  </li>
                ))}
                <li
                  key='Additional-Resource'
                  className="bg-white px-4 py-6 shadow sm:p-6 sm:rounded-lg"
                >
                  <div aria-labelledby={'service-add-on'}>
                    <div className="min-w-0 flex-1">
                      <p className="text-sm font-medium text-gray-900">
                        Dein zukünftiger Inhalt
                      </p>
                    </div>
                    <div className="mt-2 text-sm text-gray-700 space-y-4">
                      Du kennst weitere Inhalte, die hier nicht gelistet sind? Kontaktiere uns und gebe uns darüber Bescheid. Am einfachsten geht das über unseren Issue-Tracker im Open Source Code Repository der öffentlichen Verwaltung. Schau vor Deinem Feedback bitte in die <a className='text-blue-500' href='https://gitlab.opencode.de/fitko/feedback/-/issues'>Liste der bestehenden Issues</a> und erstelle erst dann <a className='text-blue-500' href='https://gitlab.opencode.de/fitko/feedback/-/issues/new'>ein neues Issue</a>.
                    </div>
                    <div className="mt-6 flex justify-end space-x-8" id="kontakt-link">
                      <div className="flex text-sm">
                        <span className="inline-flex items-center text-sm">
                          <a
                            href="#kontakt-link"
                            onMouseDown={handleMailtoEvent}
                            data-base={`${encodeEmail('fit-connect@fitko.de')}`}
                            className="inline-flex space-x-2 text-gray-400 hover:text-gray-800"
                          >
                            <span className="font-medium">Kontakt</span>
                            <IconMail
                              className="h-5 w-5"
                              aria-hidden="true"
                            />
                          </a>
                        </span>
                      </div>
                    </div>
                  </div>
                </li>
              </ul>
            </div>
          </main>
        </div>
      </div>
    </div>
  )
}

export async function getStaticProps(context) {
  const services = (await fetchServices()).map((service) => {
    // remove unwanted data and encrypt eMails
    const nosys = removeFromObjRecursive('sys', service.fields);
    const fields = replaceEmailsRecursive('eMail', nosys);
    return fields;
  })
  return {
    props: {
      services,
    },
  }
}
